@@ -388,7 +388,7 @@ void RequestsBoxController::processRequest(
 					: tr::lng_group_requests_was_added)(
 						tr::now,
 						lt_user,
-						Ui::Text::Bold(user->name),
+						Ui::Text::Bold(user->name()),
 						Ui::Text::WithEntities)
 			});
 		}
