@@ -12,7 +12,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_file_origin.h"
 #include "data/data_document.h"
 #include "data/data_session.h"
-#include "data/data_message_reactions.h"
 #include "data/data_document_media.h"
 #include "data/data_streaming.h"
 #include "data/data_peer_values.h"
@@ -58,10 +57,11 @@ constexpr auto kStarOpacityOff = 0.1;
 constexpr auto kStarOpacityOn = 1.;
 constexpr auto kStarPeriod = 3 * crl::time(1000);
 
+using Data::ReactionId;
+
 struct Descriptor {
 	PremiumPreview section = PremiumPreview::Stickers;
 	DocumentData *requestedSticker = nullptr;
-	base::flat_map<QString, ReactionDisableType> disabled;
 	bool fromSettings = false;
 	Fn<void()> hiddenCallback;
 	Fn<void(not_null<Ui::BoxContent*>)> shownCallback;
@@ -70,27 +70,9 @@ struct Descriptor {
 bool operator==(const Descriptor &a, const Descriptor &b) {
 	return (a.section == b.section)
 		&& (a.requestedSticker == b.requestedSticker)
-		&& (a.disabled == b.disabled)
 		&& (a.fromSettings == b.fromSettings);
 }
 
-[[nodiscard]] int ComputeX(int column, int columns) {
-	const auto skip = st::premiumReactionWidthSkip;
-	const auto fullWidth = columns * skip;
-	const auto left = (st::boxWideWidth - fullWidth) / 2;
-	return left + column * skip + (skip / 2);
-}
-
-[[nodiscard]] int ComputeY(int row, int rows) {
-	const auto middle = (rows > 3)
-		? (st::premiumReactionInfoTop / 2)
-		: st::premiumReactionsMiddle;
-	const auto skip = st::premiumReactionHeightSkip;
-	const auto fullHeight = rows * skip;
-	const auto top = middle - (fullHeight / 2);
-	return top + row * skip + (skip / 2);
-}
-
 struct Preload {
 	Descriptor descriptor;
 	std::shared_ptr<Data::DocumentMedia> media;
@@ -118,8 +100,10 @@ void PreloadSticker(const std::shared_ptr<Data::DocumentMedia> &media) {
 		return tr::lng_premium_summary_subtitle_voice_to_text();
 	case PremiumPreview::NoAds:
 		return tr::lng_premium_summary_subtitle_no_ads();
-	case PremiumPreview::Reactions:
-		return tr::lng_premium_summary_subtitle_unique_reactions();
+	case PremiumPreview::EmojiStatus:
+		return tr::lng_premium_summary_subtitle_emoji_status();
+	case PremiumPreview::InfiniteReactions:
+		return tr::lng_premium_summary_subtitle_infinite_reactions();
 	case PremiumPreview::Stickers:
 		return tr::lng_premium_summary_subtitle_premium_stickers();
 	case PremiumPreview::AnimatedEmoji:
@@ -144,8 +128,10 @@ void PreloadSticker(const std::shared_ptr<Data::DocumentMedia> &media) {
 		return tr::lng_premium_summary_about_voice_to_text();
 	case PremiumPreview::NoAds:
 		return tr::lng_premium_summary_about_no_ads();
-	case PremiumPreview::Reactions:
-		return tr::lng_premium_summary_about_unique_reactions();
+	case PremiumPreview::EmojiStatus:
+		return tr::lng_premium_summary_about_emoji_status();
+	case PremiumPreview::InfiniteReactions:
+		return tr::lng_premium_summary_about_infinite_reactions();
 	case PremiumPreview::Stickers:
 		return tr::lng_premium_summary_about_premium_stickers();
 	case PremiumPreview::AnimatedEmoji:
@@ -463,6 +449,8 @@ struct VideoPreviewDocument {
 		case PremiumPreview::AnimatedEmoji: return "animated_emoji";
 		case PremiumPreview::AdvancedChatManagement:
 			return "advanced_chat_management";
+		case PremiumPreview::EmojiStatus: return "emoji_status";
+		case PremiumPreview::InfiniteReactions: return "infinite_reactions";
 		case PremiumPreview::ProfileBadge: return "profile_badge";
 		case PremiumPreview::AnimatedUserpics: return "animated_userpics";
 		}
@@ -724,423 +712,12 @@ struct VideoPreviewDocument {
 	return result;
 }
 
-
-class ReactionPreview final {
-public:
-	ReactionPreview(
-		not_null<Window::SessionController*> controller,
-		const Data::Reaction &reaction,
-		ReactionDisableType type,
-		Fn<void()> update,
-		QPoint position);
-
-	[[nodiscard]] bool playsEffect() const;
-	void paint(Painter &p);
-	void paintEffect(QPainter &p);
-
-	void setOver(bool over);
-	void startAnimations();
-	void cancelAnimations();
-	[[nodiscard]] bool ready() const;
-	[[nodiscard]] bool disabled() const;
-	[[nodiscard]] QRect geometry() const;
-
-private:
-	void checkReady();
-
-	const not_null<Window::SessionController*> _controller;
-	const Fn<void()> _update;
-	const QPoint _position;
-	Ui::Animations::Simple _scale;
-	std::shared_ptr<Data::DocumentMedia> _centerMedia;
-	std::shared_ptr<Data::DocumentMedia> _aroundMedia;
-	std::unique_ptr<Lottie::SinglePlayer> _center;
-	std::unique_ptr<Lottie::SinglePlayer> _around;
-	std::unique_ptr<Ui::PathShiftGradient> _pathGradient;
-	QImage _cache1;
-	QImage _cache2;
-	bool _over = false;
-	bool _disabled = false;
-	bool _playRequested = false;
-	bool _aroundPlaying = false;
-	bool _centerPlaying = false;
-	rpl::lifetime _lifetime;
-
-};
-
-ReactionPreview::ReactionPreview(
-	not_null<Window::SessionController*> controller,
-	const Data::Reaction &reaction,
-	ReactionDisableType type,
-	Fn<void()> update,
-	QPoint position)
-: _controller(controller)
-, _update(std::move(update))
-, _position(position)
-, _centerMedia(reaction.centerIcon->createMediaView())
-, _aroundMedia(reaction.aroundAnimation->createMediaView())
-, _pathGradient(
-	HistoryView::MakePathShiftGradient(
-		controller->chatStyle(),
-		_update))
-, _disabled(type != ReactionDisableType::None) {
-	_centerMedia->checkStickerLarge();
-	_aroundMedia->checkStickerLarge();
-	checkReady();
-	if (!_center || !_around) {
-		_controller->session().downloaderTaskFinished(
-		) | rpl::take_while([=] {
-			checkReady();
-			return !_center || !_around;
-		}) | rpl::start(_lifetime);
-	}
-}
-
-QRect ReactionPreview::geometry() const {
-	const auto xsize = st::premiumReactionWidthSkip;
-	const auto ysize = st::premiumReactionHeightSkip;
-	return { _position - QPoint(xsize / 2, ysize / 2), QSize(xsize, ysize) };
-}
-
-void ReactionPreview::checkReady() {
-	const auto make = [&](
-			const std::shared_ptr<Data::DocumentMedia> &media,
-			int size) {
-		const auto bytes = media->bytes();
-		const auto filepath = media->owner()->filepath();
-		auto result = ChatHelpers::LottiePlayerFromDocument(
-			media.get(),
-			nullptr,
-			ChatHelpers::StickerLottieSize::PremiumReactionPreview,
-			QSize(size, size) * style::DevicePixelRatio(),
-			Lottie::Quality::Default);
-		result->updates() | rpl::start_with_next(_update, _lifetime);
-		return result;
-	};
-	if (!_center && _centerMedia->loaded()) {
-		_center = make(_centerMedia, st::premiumReactionSize);
-	}
-	if (!_around && _aroundMedia->loaded()) {
-		_around = make(_aroundMedia, st::premiumReactionAround);
-	}
-}
-
-void ReactionPreview::setOver(bool over) {
-	if (_over == over || _disabled) {
-		return;
-	}
-	_over = over;
-	const auto from = st::premiumReactionScale;
-	_scale.start(
-		_update,
-		over ? from : 1.,
-		over ? 1. : from,
-		st::slideWrapDuration);
-}
-
-void ReactionPreview::startAnimations() {
-	if (_disabled) {
-		return;
-	}
-	_playRequested = true;
-	if (!_center || !_center->ready() || !_around || !_around->ready()) {
-		return;
-	}
-	_update();
-}
-
-void ReactionPreview::cancelAnimations() {
-	_playRequested = false;
-}
-
-bool ReactionPreview::ready() const {
-	return _center && _center->ready();
-}
-
-bool ReactionPreview::disabled() const {
-	return _disabled;
-}
-
-void ReactionPreview::paint(Painter &p) {
-	const auto center = st::premiumReactionSize;
-	const auto scale = _scale.value(_over ? 1. : st::premiumReactionScale);
-	const auto inner = QRect(
-		-center / 2,
-		-center / 2,
-		center,
-		center
-	).translated(_position);
-	auto hq = PainterHighQualityEnabler(p);
-	const auto centerReady = _center && _center->ready();
-	const auto staticCenter = centerReady && !_centerPlaying;
-	const auto use1 = staticCenter && scale == 1.;
-	const auto use2 = staticCenter && scale == st::premiumReactionScale;
-	const auto useScale = (!use1 && !use2 && scale != 1.);
-	if (useScale) {
-		p.save();
-		p.translate(inner.center());
-		p.scale(scale, scale);
-		p.translate(-inner.center());
-	}
-	if (_disabled) {
-		p.setOpacity(kDisabledOpacity);
-	}
-	checkReady();
-	if (centerReady) {
-		if (use1 || use2) {
-			auto &cache = use1 ? _cache1 : _cache2;
-			const auto use = int(std::round(center * scale));
-			const auto rect = QRect(-use / 2, -use / 2, use, use).translated(
-				_position);
-			if (cache.isNull()) {
-				cache = _center->frame().scaledToWidth(
-					use * style::DevicePixelRatio(),
-					Qt::SmoothTransformation);
-			}
-			p.drawImage(rect, cache);
-		} else {
-			p.drawImage(inner, _center->frame());
-		}
-		if (_centerPlaying) {
-			const auto almost = (_center->frameIndex() + 1)
-				== _center->framesCount();
-			const auto marked = _center->markFrameShown();
-			if (almost && marked) {
-				_centerPlaying = false;
-			}
-		}
-		if (_around
-			&& _around->ready()
-			&& !_aroundPlaying
-			&& !_centerPlaying
-			&& _playRequested) {
-			_aroundPlaying = _centerPlaying = true;
-			_playRequested = false;
-		}
-	} else {
-		p.setBrush(_controller->chatStyle()->msgServiceBg());
-		ChatHelpers::PaintStickerThumbnailPath(
-			p,
-			_centerMedia.get(),
-			inner,
-			_pathGradient.get());
-	}
-	if (useScale) {
-		p.restore();
-	} else if (_disabled) {
-		p.setOpacity(1.);
-	}
-}
-
-bool ReactionPreview::playsEffect() const {
-	return _aroundPlaying;
-}
-
-void ReactionPreview::paintEffect(QPainter &p) {
-	if (!_aroundPlaying) {
-		return;
-	}
-	const auto size = st::premiumReactionAround;
-	const auto outer = QRect(-size/2, -size/2, size, size).translated(
-		_position);
-	const auto scale = _scale.value(_over ? 1. : st::premiumReactionScale);
-	auto hq = PainterHighQualityEnabler(p);
-	if (scale != 1.) {
-		p.save();
-		p.translate(outer.center());
-		p.scale(scale, scale);
-		p.translate(-outer.center());
-	}
-	p.drawImage(outer, _around->frame());
-	if (scale != 1.) {
-		p.restore();
-	}
-	if (_aroundPlaying) {
-		const auto almost = (_around->frameIndex() + 1)
-			== _around->framesCount();
-		const auto marked = _around->markFrameShown();
-		if (almost && marked) {
-			_aroundPlaying = false;
-		}
-	}
-}
-
-[[nodiscard]] not_null<Ui::RpWidget*> ReactionsPreview(
-		not_null<Ui::RpWidget*> parent,
-		not_null<Window::SessionController*> controller,
-		const base::flat_map<QString, ReactionDisableType> &disabled,
-		Fn<void()> readyCallback) {
-	struct State {
-		std::vector<std::unique_ptr<ReactionPreview>> entries;
-		Ui::Text::String bottom;
-		int selected = -1;
-		bool readyInvoked = false;
-	};
-	const auto result = Ui::CreateChild<Ui::RpWidget>(parent.get());
-	result->show();
-
-	auto &lifetime = result->lifetime();
-	const auto state = lifetime.make_state<State>();
-
-	result->setMouseTracking(true);
-
-	parent->sizeValue(
-	) | rpl::start_with_next([=] {
-		result->setGeometry(parent->rect());
-	}, result->lifetime());
-
-	using namespace HistoryView;
-	const auto list = controller->session().data().reactions().list(
-		Data::Reactions::Type::Active);
-	const auto count = ranges::count(list, true, &Data::Reaction::premium);
-	const auto rows = (count + kReactionsPerRow - 1) / kReactionsPerRow;
-	const auto inrowmax = (count + rows - 1) / rows;
-	const auto inrowless = (inrowmax * rows - count);
-	const auto inrowmore = rows - inrowless;
-	const auto inmaxrows = inrowmore * inrowmax;
-	auto index = 0;
-	auto disableType = ReactionDisableType::None;
-	for (const auto &reaction : list) {
-		if (!reaction.premium) {
-			continue;
-		}
-		const auto inrow = (index < inmaxrows) ? inrowmax : (inrowmax - 1);
-		const auto row = (index < inmaxrows)
-			? (index / inrow)
-			: (inrowmore + ((index - inmaxrows) / inrow));
-		const auto column = (index < inmaxrows)
-			? (index % inrow)
-			: ((index - inmaxrows) % inrow);
-		++index;
-		if (!reaction.centerIcon || !reaction.aroundAnimation) {
-			continue;
-		}
-		const auto i = disabled.find(reaction.emoji);
-		const auto disable = (i != end(disabled))
-			? i->second
-			: ReactionDisableType::None;
-		if (disable != ReactionDisableType::None) {
-			disableType = disable;
-		}
-		state->entries.push_back(std::make_unique<ReactionPreview>(
-			controller,
-			reaction,
-			disable,
-			[=] { result->update(); },
-			QPoint(ComputeX(column, inrow), ComputeY(row, rows))));
-	}
-
-	const auto bottom1 = tr::lng_reaction_premium_info(tr::now);
-	const auto bottom2 = (disableType == ReactionDisableType::None)
-		? QString()
-		: (disableType == ReactionDisableType::Group)
-		? tr::lng_reaction_premium_no_group(tr::now)
-		: tr::lng_reaction_premium_no_channel(tr::now);
-	state->bottom.setText(
-		st::defaultTextStyle,
-		(bottom1 + '\n' + bottom2).trimmed());
-
-	result->paintRequest(
-	) | rpl::start_with_next([=] {
-		auto p = Painter(result);
-		auto effects = std::vector<Fn<void()>>();
-		auto ready = 0;
-		for (const auto &entry : state->entries) {
-			entry->paint(p);
-			if (entry->ready()) {
-				++ready;
-			}
-			if (entry->playsEffect()) {
-				effects.push_back([&] {
-					entry->paintEffect(p);
-				});
-			}
-		}
-		if (!state->readyInvoked
-			&& readyCallback
-			&& ready > 0
-			&& ready == state->entries.size()) {
-			state->readyInvoked = true;
-			readyCallback();
-
-		}
-		const auto padding = st::boxRowPadding;
-		const auto available = parent->width()
-			- padding.left()
-			- padding.right();
-		const auto top = st::premiumReactionInfoTop
-			+ ((state->bottom.maxWidth() > available)
-				? st::normalFont->height
-				: 0);
-		p.setPen(st::premiumButtonFg);
-		state->bottom.draw(
-			p,
-			padding.left(),
-			top,
-			available,
-			style::al_top);
-		for (const auto &paint : effects) {
-			paint();
-		}
-	}, lifetime);
-
-	const auto lookup = [=](QPoint point) {
-		auto index = 0;
-		for (const auto &entry : state->entries) {
-			if (entry->geometry().contains(point) && !entry->disabled()) {
-				return index;
-			}
-			++index;
-		}
-		return -1;
-	};
-	const auto select = [=](int index) {
-		const auto wasInside = (state->selected >= 0);
-		const auto nowInside = (index >= 0);
-		if (state->selected != index) {
-			if (wasInside) {
-				state->entries[state->selected]->setOver(false);
-			}
-			if (nowInside) {
-				state->entries[index]->setOver(true);
-			}
-			state->selected = index;
-		}
-		if (wasInside != nowInside) {
-			result->setCursor(nowInside
-				? style::cur_pointer
-				: style::cur_default);
-		}
-	};
-	result->events(
-	) | rpl::start_with_next([=](not_null<QEvent*> event) {
-		if (event->type() == QEvent::MouseButtonPress) {
-			const auto point = static_cast<QMouseEvent*>(event.get())->pos();
-			if (state->selected >= 0) {
-				state->entries[state->selected]->cancelAnimations();
-			}
-			if (const auto index = lookup(point); index >= 0) {
-				state->entries[index]->startAnimations();
-			}
-		} else if (event->type() == QEvent::MouseMove) {
-			const auto point = static_cast<QMouseEvent*>(event.get())->pos();
-			select(lookup(point));
-		} else if (event->type() == QEvent::Leave) {
-			select(-1);
-		}
-	}, lifetime);
-
-	return result;
-}
-
 [[nodiscard]] not_null<Ui::RpWidget*> GenerateDefaultPreview(
 		not_null<Ui::RpWidget*> parent,
 		not_null<Window::SessionController*> controller,
 		PremiumPreview section,
 		Fn<void()> readyCallback) {
 	switch (section) {
-	case PremiumPreview::Reactions:
-		return ReactionsPreview(parent, controller, {}, readyCallback);
 	case PremiumPreview::Stickers:
 		return StickersPreview(parent, controller, readyCallback);
 	default:
@@ -1226,8 +803,6 @@ void PreviewBox(
 		Ui::RpWidget *content = nullptr;
 		Ui::RpWidget *stickersPreload = nullptr;
 		bool stickersPreloadReady = false;
-		Ui::RpWidget *reactionsPreload = nullptr;
-		bool reactionsPreloadReady = false;
 		bool preloadScheduled = false;
 		bool showFinished = false;
 		Ui::Animations::Simple animation;
@@ -1290,21 +865,6 @@ void PreviewBox(
 				ready);
 			state->stickersPreload->hide();
 		}
-		if (now != PremiumPreview::Reactions && !state->reactionsPreload) {
-			const auto ready = [=] {
-				if (state->reactionsPreload) {
-					state->reactionsPreloadReady = true;
-				} else {
-					state->preload();
-				}
-			};
-			state->reactionsPreload = GenerateDefaultPreview(
-				outer,
-				controller,
-				PremiumPreview::Reactions,
-				ready);
-			state->reactionsPreload->hide();
-		}
 	};
 
 	switch (descriptor.section) {
@@ -1313,13 +873,6 @@ void PreviewBox(
 			? StickerPreview(outer, controller, media, state->preload)
 			: StickersPreview(outer, controller, state->preload);
 		break;
-	case PremiumPreview::Reactions:
-		state->content = ReactionsPreview(
-			outer,
-			controller,
-			descriptor.disabled,
-			state->preload);
-		break;
 	default:
 		state->content = GenericPreview(
 			outer,
@@ -1379,13 +932,6 @@ void PreviewBox(
 			if (base::take(state->stickersPreloadReady)) {
 				state->preload();
 			}
-		} else if (now == PremiumPreview::Reactions
-			&& state->reactionsPreload) {
-			state->content = base::take(state->reactionsPreload);
-			state->content->show();
-			if (base::take(state->reactionsPreloadReady)) {
-				state->preload();
-			}
 		} else {
 			state->content = GenerateDefaultPreview(
 				outer,
@@ -1455,12 +1001,14 @@ void PreviewBox(
 		};
 		auto unlock = state->selected.value(
 		) | rpl::map([=](PremiumPreview section) {
-			return (section == PremiumPreview::Reactions)
+			return (section == PremiumPreview::InfiniteReactions)
 				? tr::lng_premium_unlock_reactions()
 				: (section == PremiumPreview::Stickers)
 				? tr::lng_premium_unlock_stickers()
 				: (section == PremiumPreview::AnimatedEmoji)
 				? tr::lng_premium_unlock_emoji()
+				: (section == PremiumPreview::EmojiStatus)
+				? tr::lng_premium_unlock_status()
 				: tr::lng_premium_more_about();
 		}) | rpl::flatten_latest();
 		auto button = descriptor.fromSettings
@@ -1623,11 +1171,9 @@ void ShowStickerPreviewBox(
 void ShowPremiumPreviewBox(
 		not_null<Window::SessionController*> controller,
 		PremiumPreview section,
-		const base::flat_map<QString, ReactionDisableType> &disabled,
 		Fn<void(not_null<Ui::BoxContent*>)> shown) {
 	Show(controller, Descriptor{
 		.section = section,
-		.disabled = disabled,
 		.shownCallback = std::move(shown),
 	});
 }
@@ -1670,14 +1216,14 @@ void DoubledLimitsPreviewBox(
 		});
 	}
 	{
-		const auto premium = limits.dialogsFolderPinnedPremium();
+		const auto premium = limits.dialogsPinnedPremium();
 		entries.push_back(Ui::Premium::ListEntry{
 			tr::lng_premium_double_limits_subtitle_pins(),
 			tr::lng_premium_double_limits_about_pins(
 				lt_count,
 				rpl::single(float64(premium)),
 				Ui::Text::RichLangValue),
-			limits.dialogsFolderPinnedDefault(),
+			limits.dialogsPinnedDefault(),
 			premium,
 		});
 	}
