@@ -289,7 +289,7 @@ void RenameBox(not_null<Ui::GenericBox*> box) {
 		Unexpected("Type in LottieForType.");
 	}();
 	const auto size = st::sessionBigLottieSize;
-	return std::make_unique<Lottie::Icon>(Lottie::IconDescriptor{
+	return Lottie::MakeIcon({
 		.path = u":/icons/settings/devices/"_q + path + u".lottie"_q,
 		.sizeOverride = QSize(size, size),
 	});
@@ -1163,6 +1163,10 @@ Sessions::Sessions(
 	setupContent(controller);
 }
 
+rpl::producer<QString> Sessions::title() {
+	return tr::lng_settings_sessions_title();
+}
+
 void Sessions::setupContent(not_null<Window::SessionController*> controller) {
 	const auto container = Ui::CreateChild<Ui::VerticalLayout>(this);
 	AddSkip(container, st::settingsPrivacySkip);
