@@ -108,7 +108,16 @@ public:
 		Fn<void()> updateCallback) override;
 	void rightActionStopLastRipple() override;
 
-	int nameIconWidth() const override {
+	int paintNameIconGetWidth(
+			Painter &p,
+			Fn<void()> repaint,
+			crl::time now,
+			int nameLeft,
+			int nameTop,
+			int nameWidth,
+			int availableWidth,
+			int outerWidth,
+			bool selected) override {
 		return 0;
 	}
 	QSize rightActionSize() const override {
