@@ -15,7 +15,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "main/main_session.h"
 #include "window/window_session_controller.h"
 #include "ui/cached_round_corners.h"
-#include "facades.h"
+#include "api/api_bot.h"
 #include "styles/style_widgets.h"
 #include "styles/style_chat.h"
 
@@ -169,7 +169,9 @@ void BotKeyboard::leaveEventHook(QEvent *e) {
 	clearSelection();
 }
 
-bool BotKeyboard::moderateKeyActivate(int key) {
+bool BotKeyboard::moderateKeyActivate(
+		int key,
+		Fn<ClickContext(FullMsgId)> context) {
 	const auto &data = _controller->session().data();
 
 	const auto botCommand = [](int key) {
@@ -202,7 +204,11 @@ bool BotKeyboard::moderateKeyActivate(int key) {
 				if (!markup->data.rows.empty()
 					&& index >= 0
 					&& index < int(markup->data.rows.front().size())) {
-					App::activateBotCommand(_controller, item, 0, index);
+					Api::ActivateBotCommand(
+						context(
+							_wasForMsgId).other.value<ClickHandlerContext>(),
+						0,
+						index);
 					return true;
 				}
 			} else if (const auto user = item->history()->peer->asUser()) {
