@@ -7,15 +7,11 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
-#include "core/core_settings.h"
 #include "mtproto/mtproto_auth_key.h"
 #include "mtproto/mtproto_proxy_data.h"
 #include "base/timer.h"
 
-class MainWindow;
-class MainWidget;
-class FileUploader;
-class Translator;
+class History;
 
 namespace Platform {
 class Integration;
@@ -69,6 +65,7 @@ class Instance;
 } // namespace Audio
 namespace View {
 class OverlayWidget;
+struct OpenRequest;
 } // namespace View
 namespace Player {
 class FloatController;
@@ -103,6 +100,8 @@ namespace Core {
 
 class Launcher;
 struct LocalUrlHandler;
+class Settings;
+class Tray;
 
 enum class LaunchState {
 	Running,
@@ -147,6 +146,9 @@ public:
 	[[nodiscard]] Data::DownloadManager &downloadManager() const {
 		return *_downloadManager;
 	}
+	[[nodiscard]] Tray &tray() const {
+		return *_tray;
+	}
 
 	// Windows interface.
 	bool hasActiveWindow(not_null<Main::Session*> session) const;
@@ -158,14 +160,17 @@ public:
 	Window::Controller *ensureSeparateWindowForPeer(
 		not_null<PeerData*> peer,
 		MsgId showAtMsgId);
+	void closeWindow(not_null<Window::Controller*> window);
+	void windowActivated(not_null<Window::Controller*> window);
 	bool closeActiveWindow();
 	bool minimizeActiveWindow();
 	[[nodiscard]] QWidget *getFileDialogParent();
 	void notifyFileDialogShown(bool shown);
 	void checkSystemDarkMode();
+	[[nodiscard]] bool isActiveForTrayMenu() const;
+	void closeChatFromWindows(not_null<PeerData*> peer);
 
 	// Media view interface.
-	void checkMediaViewActivation();
 	bool hideMediaView();
 
 	[[nodiscard]] QPoint getPointForCallPanelCenter() const;
@@ -202,7 +207,7 @@ public:
 	[[nodiscard]] bool exportPreventsQuit();
 
 	// Main::Session component.
-	Main::Session *maybeActiveSession() const;
+	Main::Session *maybePrimarySession() const;
 	[[nodiscard]] int unreadBadge() const;
 	[[nodiscard]] bool unreadBadgeMuted() const;
 	[[nodiscard]] rpl::producer<> unreadBadgeChanges() const;
@@ -316,6 +321,11 @@ private:
 	void startDomain();
 	void startEmojiImageLoader();
 	void startSystemDarkModeViewer();
+	void startTray();
+
+	void enumerateWindows(
+		Fn<void(not_null<Window::Controller*>)> callback) const;
+	void processSecondaryWindow(not_null<Window::Controller*> window);
 
 	friend void QuitAttempt();
 	void quitDelayed();
@@ -376,6 +386,8 @@ private:
 	std::unique_ptr<Lang::Translator> _translator;
 	QPointer<Ui::BoxContent> _badProxyDisableBox;
 
+	const std::unique_ptr<Tray> _tray;
+
 	std::unique_ptr<Media::Player::FloatController> _floatPlayers;
 	Media::Player::FloatDelegate *_defaultFloatPlayerDelegate = nullptr;
 	Media::Player::FloatDelegate *_replacementFloatPlayerDelegate = nullptr;
@@ -394,6 +406,8 @@ private:
 	};
 	base::flat_map<not_null<QWidget*>, LeaveFilter> _leaveFilters;
 
+	rpl::event_stream<Media::View::OpenRequest> _openInMediaViewRequests;
+
 	rpl::lifetime _lifetime;
 
 	crl::time _lastNonIdleTime = 0;
