@@ -35,6 +35,7 @@ namespace Data {
 class Session;
 class GroupCall;
 class CloudImageView;
+struct ReactionId;
 
 int PeerColorIndex(PeerId peerId);
 int PeerColorIndex(BareId bareId);
@@ -100,6 +101,22 @@ bool ApplyBotMenuButton(
 	not_null<BotInfo*> info,
 	const MTPBotMenuButton *button);
 
+enum class AllowedReactionsType {
+	All,
+	Default,
+	Some,
+};
+
+struct AllowedReactions {
+	std::vector<ReactionId> some;
+	AllowedReactionsType type = AllowedReactionsType::Some;
+};
+
+bool operator<(const AllowedReactions &a, const AllowedReactions &b);
+bool operator==(const AllowedReactions &a, const AllowedReactions &b);
+
+[[nodiscard]] AllowedReactions Parse(const MTPChatReactions &value);
+
 } // namespace Data
 
 class PeerClickHandler : public ClickHandler {
@@ -184,7 +201,7 @@ public:
 		return _notify.change(settings);
 	}
 	bool notifyChange(
-			std::optional<int> muteForSeconds,
+			Data::MuteValue muteForSeconds,
 			std::optional<bool> silentPosts,
 			std::optional<Data::NotifySound> sound) {
 		return _notify.change(muteForSeconds, silentPosts, sound);
