@@ -1014,7 +1014,9 @@ void Widget::handleSongChange() {
 	TextWithEntities textWithEntities;
 	if (document->isVoiceMessage() || document->isVideoMessage()) {
 		if (const auto item = document->owner().message(current.contextId())) {
-			const auto name = item->fromOriginal()->name;
+			const auto name = (!item->out() || item->isPost())
+				? item->fromOriginal()->name()
+				: tr::lng_from_you(tr::now);
 			const auto date = [item] {
 				const auto parsed = ItemDateTime(item);
 				const auto date = parsed.date();
