@@ -30,8 +30,6 @@ public:
 
 	void updateWindowIcon() override;
 
-	void psShowTrayMenu();
-
 	bool preventsQuit(Core::QuitReason reason) override;
 
 	class Private;
@@ -39,25 +37,12 @@ public:
 protected:
 	bool eventFilter(QObject *obj, QEvent *evt) override;
 
-	void handleActiveChangedHook() override;
 	void stateChangedHook(Qt::WindowState state) override;
 	void initHook() override;
 	void unreadCounterChangedHook() override;
 
-	bool hasTrayIcon() const override {
-		return trayIcon;
-	}
-
 	void updateGlobalMenuHook() override;
 
-	void workmodeUpdated(Core::Settings::WorkMode mode) override;
-
-	QSystemTrayIcon *trayIcon = nullptr;
-	QMenu *trayIconMenu = nullptr;
-
-	void psTrayMenuUpdated();
-	void psSetupTrayIcon();
-
 	void closeWithoutDestroy() override;
 	void createGlobalMenu() override;
 
@@ -66,7 +51,6 @@ private:
 
 	void hideAndDeactivate();
 	void updateIconCounters();
-	[[nodiscard]] QIcon generateIconForTray(int counter, bool muted) const;
 
 	std::unique_ptr<Private> _private;
 
