@@ -570,7 +570,8 @@ void ConfirmContactBox::paintEvent(QPaintEvent *e) {
 	auto context = theme->preparePaintContext(
 		_chatStyle.get(),
 		rect(),
-		rect());
+		rect(),
+		controller()->isGifPausedAtLeastFor(Window::GifPauseReason::Layer));
 	p.translate(st::boxPadding.left(), 0);
 	if (_comment) {
 		context.outbg = _comment->hasOutLayout();
