@@ -760,7 +760,7 @@ void Editor::exportTheme() {
 			_window->show(Ui::MakeInformBox(tr::lng_theme_editor_error()));
 			return;
 		}
-		Ui::Toast::Show(tr::lng_theme_editor_done(tr::now));
+		_window->showToast(tr::lng_theme_editor_done(tr::now));
 	}));
 }
 
@@ -817,10 +817,10 @@ QByteArray Editor::ColorizeInContent(
 
 void Editor::save() {
 	if (Core::App().passcodeLocked()) {
-		Ui::Toast::Show(tr::lng_theme_editor_need_unlock(tr::now));
+		_window->showToast(tr::lng_theme_editor_need_unlock(tr::now));
 		return;
 	} else if (!_window->account().sessionExists()) {
-		Ui::Toast::Show(tr::lng_theme_editor_need_auth(tr::now));
+		_window->showToast(tr::lng_theme_editor_need_auth(tr::now));
 		return;
 	} else if (_saving) {
 		return;
