@@ -13,7 +13,6 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/edit_color_box.h"
 #include "lang/lang_keys.h"
 #include "base/call_delayed.h"
-#include "base/qt_adapters.h"
 
 namespace Window {
 namespace Theme {
@@ -160,7 +159,7 @@ void EditorBlock::Row::fillSearchIndex() {
 		+ ' ' + _valueString;
 	const auto words = toIndex.toLower().split(
 		SearchSplitter,
-		base::QStringSkipEmptyParts);
+		Qt::SkipEmptyParts);
 	for (const auto &word : words) {
 		_searchWords.emplace(word);
 		_searchStartChars.emplace(word[0]);
