@@ -137,9 +137,9 @@ void Manager::sendSignalingAsync(int delayMs, int cause) {
 		}
 	};
 	if (delayMs) {
-		_thread->PostDelayedTask(RTC_FROM_HERE, std::move(task), delayMs);
+		_thread->PostDelayedTask(std::move(task), delayMs);
 	} else {
-		_thread->PostTask(RTC_FROM_HERE, std::move(task));
+		_thread->PostTask(std::move(task));
 	}
 }
 
@@ -147,7 +147,7 @@ void Manager::start() {
 	const auto weak = std::weak_ptr<Manager>(shared_from_this());
 	const auto thread = _thread;
 	const auto sendSignalingMessage = [=](Message &&message) {
-		thread->PostTask(RTC_FROM_HERE, [=, message = std::move(message)]() mutable {
+		thread->PostTask([=, message = std::move(message)]() mutable {
 			const auto strong = weak.lock();
 			if (!strong) {
 				return;
@@ -165,7 +165,7 @@ void Manager::start() {
 			rtcServers,
             std::move(proxy),
 			[=](const NetworkManager::State &state) {
-				thread->PostTask(RTC_FROM_HERE, [=] {
+				thread->PostTask([=] {
 					const auto strong = weak.lock();
 					if (!strong) {
 						return;
@@ -198,7 +198,7 @@ void Manager::start() {
 				});
 			},
 			[=](DecryptedMessage &&message) {
-				thread->PostTask(RTC_FROM_HERE, [=, message = std::move(message)]() mutable {
+				thread->PostTask([=, message = std::move(message)]() mutable {
 					if (const auto strong = weak.lock()) {
 						strong->receiveMessage(std::move(message));
 					}
@@ -214,9 +214,9 @@ void Manager::start() {
 					}
 				};
 				if (delayMs) {
-					thread->PostDelayedTask(RTC_FROM_HERE, task, delayMs);
+					thread->PostDelayedTask(task, delayMs);
 				} else {
-					thread->PostTask(RTC_FROM_HERE, task);
+					thread->PostTask(task);
 				}
 			});
 	}));
@@ -230,7 +230,7 @@ void Manager::start() {
 			videoCapture,
 			sendSignalingMessage,
 			[=](Message &&message) {
-				thread->PostTask(RTC_FROM_HERE, [=, message = std::move(message)]() mutable {
+				thread->PostTask([=, message = std::move(message)]() mutable {
 					const auto strong = weak.lock();
 					if (!strong) {
 						return;
@@ -359,7 +359,7 @@ void Manager::getNetworkStats(std::function<void (TrafficStats, CallStats)> comp
         CallStats callStats;
         networkManager->fillCallStats(callStats);
 
-        thread->PostTask(RTC_FROM_HERE, [weak, networkStats, completion = std::move(completion), callStats = std::move(callStats), statsLogPath = statsLogPath] {
+        thread->PostTask([weak, networkStats, completion = std::move(completion), callStats = std::move(callStats), statsLogPath = statsLogPath] {
             const auto strong = weak.lock();
             if (!strong) {
                 return;
