@@ -196,6 +196,8 @@ protected:
 	QImage prepareRippleMask() const override;
 	QPoint prepareRippleStartPosition() const override;
 
+	[[nodiscard]] float64 iconOverOpacity() const;
+
 private:
 	const style::IconButton &_st;
 	const style::icon *_iconOverride = nullptr;
@@ -261,7 +263,9 @@ public:
 		const style::SettingsButton &st);
 	~SettingsButton();
 
-	SettingsButton *toggleOn(rpl::producer<bool> &&toggled);
+	SettingsButton *toggleOn(
+		rpl::producer<bool> &&toggled,
+		bool ignoreClick = false);
 	bool toggled() const;
 	rpl::producer<bool> toggledChanges() const;
 	rpl::producer<bool> toggledValue() const;
