@@ -24,7 +24,7 @@ MenuCallback CreateAddActionCallback(
 				a.icon);
 			// Dummy menu.
 			action->setMenu(Ui::CreateChild<QMenu>(menu->menu().get()));
-			a.fillSubmenu(menu->ensureSubmenu(action));
+			a.fillSubmenu(menu->ensureSubmenu(action, menu->st()));
 			return action;
 		} else if (a.isSeparator) {
 			return menu->addSeparator();
