@@ -12,9 +12,6 @@
 #include "base/const_string.h"
 #include "base/integration.h"
 
-#include <QtCore/QUrl>
-#include <QtGui/QDesktopServices>
-
 #include <giomm.h>
 
 namespace Webview::WebKit2Gtk {
@@ -127,7 +124,7 @@ private:
 		const Glib::ustring &object_path,
 		const Glib::ustring &interface_name,
 		const Glib::ustring &method_name,
-		const Glib::VariantContainerBase &parameters,
+		Glib::VariantContainerBase parameters,
 		const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation);
 
 	bool _debug = false;
@@ -213,10 +210,10 @@ void Instance::create() {
 				std::string(kInterface),
 				"Create",
 				{});
-
-			return;
 		} catch (...) {
 		}
+
+		return;
 	}
 
 	if (!resolve()) {
@@ -400,10 +397,8 @@ bool Instance::decidePolicy(
 					const Glib::ustring &signal_name,
 					Glib::VariantContainerBase parameters) {
 					try {
-						auto parametersCopy = parameters;
-
 						result = base::Platform::GlibVariantCast<
-							bool>(parametersCopy.get_child(0));
+							bool>(parameters.get_child(0));
 					} catch (...) {
 					}
 
@@ -483,12 +478,10 @@ bool Instance::scriptDialog(WebKitScriptDialog *dialog) {
 					const Glib::ustring &signal_name,
 					Glib::VariantContainerBase parameters) {
 					try {
-						auto parametersCopy = parameters;
-
 						accepted = base::Platform::GlibVariantCast<
-							bool>(parametersCopy.get_child(0));
+							bool>(parameters.get_child(0));
 						result = base::Platform::GlibVariantCast<
-							Glib::ustring>(parametersCopy.get_child(1));
+							Glib::ustring>(parameters.get_child(1));
 					} catch (...) {
 					}
 
@@ -555,6 +548,8 @@ bool Instance::resolve() {
 			return true;
 		} catch (...) {
 		}
+
+		return false;
 	}
 
 	return Resolve();
@@ -811,7 +806,7 @@ void Instance::startProcess() {
 
 	// timeout in case something goes wrong
 	const auto timeout = Glib::TimeoutSource::create(5000);
-	timeout->connect([=] {
+	timeout->connect([&] {
 		if (loop->is_running()) {
 			loop->quit();
 		}
@@ -820,6 +815,7 @@ void Instance::startProcess() {
 	timeout->attach();
 
 	loop->run();
+	timeout->destroy();
 
 	_dbusConnection = [&] {
 		try {
@@ -854,10 +850,8 @@ void Instance::connectToRemoteSignals() {
 			const Glib::ustring &signal_name,
 			Glib::VariantContainerBase parameters) {
 			try {
-				auto parametersCopy = parameters;
-
 				const auto message = base::Platform::GlibVariantCast<
-					std::string>(parametersCopy.get_child(0));
+					std::string>(parameters.get_child(0));
 
 				_messageHandler(message);
 			} catch (...) {
@@ -878,18 +872,18 @@ void Instance::connectToRemoteSignals() {
 				const Glib::ustring &signal_name,
 				Glib::VariantContainerBase parameters) {
 				try {
-					auto parametersCopy = parameters;
-
 					const auto uri = base::Platform::GlibVariantCast<
-						Glib::ustring>(parametersCopy.get_child(0));
+						Glib::ustring>(parameters.get_child(0));
 					const auto newWindow = base::Platform::GlibVariantCast<
-						bool>(parametersCopy.get_child(1));
+						bool>(parameters.get_child(1));
 					const auto result = [&] {
 						if (newWindow) {
 							if (_navigationStartHandler
 								&& _navigationStartHandler(uri, true)) {
-								QDesktopServices::openUrl(
-									QString::fromUtf8(uri.c_str()));
+								try {
+									Gio::AppInfo::launch_default_for_uri(uri);
+								} catch (...) {
+								}
 							}
 							return false;
 						}
@@ -924,10 +918,8 @@ void Instance::connectToRemoteSignals() {
 				const Glib::ustring &signal_name,
 				Glib::VariantContainerBase parameters) {
 				try {
-					auto parametersCopy = parameters;
-
 					const auto success = base::Platform::GlibVariantCast<
-						bool>(parametersCopy.get_child(0));
+						bool>(parameters.get_child(0));
 
 					_navigationDoneHandler(success);
 				} catch (...) {
@@ -948,14 +940,12 @@ void Instance::connectToRemoteSignals() {
 				const Glib::ustring &signal_name,
 				Glib::VariantContainerBase parameters) {
 				try {
-					auto parametersCopy = parameters;
-
 					const auto type = base::Platform::GlibVariantCast<
-						int>(parametersCopy.get_child(0));
+						int>(parameters.get_child(0));
 					const auto text = base::Platform::GlibVariantCast<
-						Glib::ustring>(parametersCopy.get_child(1));
+						Glib::ustring>(parameters.get_child(1));
 					const auto value = base::Platform::GlibVariantCast<
-						Glib::ustring>(parametersCopy.get_child(2));
+						Glib::ustring>(parameters.get_child(2));
 
 					const auto dialogType = (type == WEBKIT_SCRIPT_DIALOG_PROMPT)
 						? DialogType::Prompt
@@ -1043,11 +1033,9 @@ void Instance::handleMethodCall(
 		const Glib::ustring &object_path,
 		const Glib::ustring &interface_name,
 		const Glib::ustring &method_name,
-		const Glib::VariantContainerBase &parameters,
+		Glib::VariantContainerBase parameters,
 		const Glib::RefPtr<Gio::DBus::MethodInvocation> &invocation) {
 	try {
-		auto parametersCopy = parameters;
-
 		if (method_name == "Create") {
 			create();
 			invocation->return_value({});
@@ -1068,7 +1056,7 @@ void Instance::handleMethodCall(
 			}
 		} else if (method_name == "Navigate") {
 			const auto url = base::Platform::GlibVariantCast<
-				Glib::ustring>(parametersCopy.get_child(0));
+				Glib::ustring>(parameters.get_child(0));
 
 			navigate(url);
 			invocation->return_value({});
@@ -1079,14 +1067,14 @@ void Instance::handleMethodCall(
 			return;
 		} else if (method_name == "Init") {
 			const auto js = base::Platform::GlibVariantCast<
-				std::string>(parametersCopy.get_child(0));
+				std::string>(parameters.get_child(0));
 
 			init(js);
 			invocation->return_value({});
 			return;
 		} else if (method_name == "Eval") {
 			const auto js = base::Platform::GlibVariantCast<
-				std::string>(parametersCopy.get_child(0));
+				std::string>(parameters.get_child(0));
 
 			eval(js);
 			invocation->return_value({});
